@@ -14,8 +14,6 @@ ISO 32000-2:2020 (PDF 2.0)
 from enum import IntFlag, auto
 from typing import Dict, Tuple
 
-from ._utils import classproperty, deprecate_with_replacement
-
 
 class Core:
     """Keywords that don't quite belong anywhere else."""
@@ -33,6 +31,7 @@ class TrailerKeys:
     ID = "/ID"
     INFO = "/Info"
     SIZE = "/Size"
+    PREV = "/Prev"
 
 
 class CatalogAttributes:
@@ -161,46 +160,6 @@ class Ressources:  # deprecated
     .. deprecated:: 5.0.0
     """
 
-    @classproperty
-    def EXT_G_STATE(cls) -> str:  # noqa: N805
-        deprecate_with_replacement("Ressources", "Resources", "5.0.0")
-        return "/ExtGState"  # dictionary, optional
-
-    @classproperty
-    def COLOR_SPACE(cls) -> str:  # noqa: N805
-        deprecate_with_replacement("Ressources", "Resources", "5.0.0")
-        return "/ColorSpace"  # dictionary, optional
-
-    @classproperty
-    def PATTERN(cls) -> str:  # noqa: N805
-        deprecate_with_replacement("Ressources", "Resources", "5.0.0")
-        return "/Pattern"  # dictionary, optional
-
-    @classproperty
-    def SHADING(cls) -> str:  # noqa: N805
-        deprecate_with_replacement("Ressources", "Resources", "5.0.0")
-        return "/Shading"  # dictionary, optional
-
-    @classproperty
-    def XOBJECT(cls) -> str:  # noqa: N805
-        deprecate_with_replacement("Ressources", "Resources", "5.0.0")
-        return "/XObject"  # dictionary, optional
-
-    @classproperty
-    def FONT(cls) -> str:  # noqa: N805
-        deprecate_with_replacement("Ressources", "Resources", "5.0.0")
-        return "/Font"  # dictionary, optional
-
-    @classproperty
-    def PROC_SET(cls) -> str:  # noqa: N805
-        deprecate_with_replacement("Ressources", "Resources", "5.0.0")
-        return "/ProcSet"  # array, optional
-
-    @classproperty
-    def PROPERTIES(cls) -> str:  # noqa: N805
-        deprecate_with_replacement("Ressources", "Resources", "5.0.0")
-        return "/Properties"  # dictionary, optional
-
 
 class PagesAttributes:
     """§7.7.3.2 of the 1.7 and 2.0 reference."""
@@ -217,13 +176,15 @@ class PageAttributes:
 
     TYPE = "/Type"  # name, required; must be /Page
     PARENT = "/Parent"  # dictionary, required; a pages object
-    LAST_MODIFIED = "/LastModified"  # date, optional; date and time of last modification
+    LAST_MODIFIED = (
+        "/LastModified"  # date, optional; date and time of last modification
+    )
     RESOURCES = "/Resources"  # dictionary, required if there are any
     MEDIABOX = "/MediaBox"  # rectangle, required; rectangle specifying page size
-    CROPBOX = "/CropBox"  # rectangle, optional; rectangle
-    BLEEDBOX = "/BleedBox"  # rectangle, optional; rectangle
-    TRIMBOX = "/TrimBox"  # rectangle, optional; rectangle
-    ARTBOX = "/ArtBox"  # rectangle, optional; rectangle
+    CROPBOX = "/CropBox"  # rectangle, optional
+    BLEEDBOX = "/BleedBox"  # rectangle, optional
+    TRIMBOX = "/TrimBox"  # rectangle, optional
+    ARTBOX = "/ArtBox"  # rectangle, optional
     BOX_COLOR_INFO = "/BoxColorInfo"  # dictionary, optional
     CONTENTS = "/Contents"  # stream or array, optional
     ROTATE = "/Rotate"  # integer, optional; page rotation in degrees
